#!/usr/bin/env python
# -*- coding: UTF-8 -*-
#
# Copyright © 2009 Leandro Regueiro Iglesias
#
#This program is free software: you can redistribute it and/or modify
#it under the terms of the GNU General Public License as published by
#the Free Software Foundation, either version 2 of the License, or
#(at your option) any later version.
#
#This program is distributed in the hope that it will be useful,
#but WITHOUT ANY WARRANTY; without even the implied warranty of
#MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#GNU General Public License for more details.
#
#You should have received a copy of the GNU General Public License
#along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
#----------------------------------------------------------------------
#
#Este é un programa de software libre e, xa que logo, vostede pode 
#redistribuílo e/ou modificalo consonte os termos establecidos na licenza 
#pública xeral de GNU segundo a súa publicación pola Free Software Foundation. 
#Pode escoller a versión 2 da licenza ou, se prefire, calquera outra versión 
#posterior.
#
#Este programa distribúese coa esperanza de que resulte útil, se ben NON TEN 
#NINGUNHA GARANTÍA, nin sequera a garantía implícita de COMERCIABILIDADE ou 
#IDONEIDADE PARA UN DETERMINADO FIN.  Para obter información máis detallada 
#pode consultar a licenza pública xeral de GNU.
#
#Dáse por sentado que recibiu unha copia da licenza pública xeral de GNU xunto
#con este programa. Se non for así, vexa <http://www.gnu.org/licenses/>.

from xml.dom import minidom
import os, sys, time, sqlite3

DB_PATH = os.getcwd()


def crear_bd():# crea a estrutura da base de datos
    conexion = sqlite3.connect(DB_PATH + '/bd_glosarios_galego.bd')
    cursor = conexion.cursor()
    cursor.execute('''CREATE TABLE concepto (id_concepto INTEGER PRIMARY KEY, ambito TEXT, definicion TEXT, exemplo TEXT, nome_glosario TEXT)''')
    conexion.commit()
    cursor.execute('''CREATE TABLE traducion (id_traducion INTEGER PRIMARY KEY, texto TEXT, idioma TEXT, is_recommended TEXT, partOfSpeech TEXT, grammaticalGender TEXT, grammaticalNumber TEXT, id_concepto INTEGER, FOREIGN KEY (id_concepto) REFERENCES concepto(id_concepto) ON UPDATE CASCADE)''')
    conexion.commit()
    cursor.close()
    conexion.close()



def obterMaximo(cursor, nome):# para obter o último idConcepto ou idTraducion
    consulta = "SELECT max(id_" + nome + ") FROM " + nome
    cursor.execute(consulta)
    dato = cursor.fetchone()[0]
    if (dato == None):
        return 0
    else:
        return int(dato)



def meterDatos(ficheiroTBX, nomeGlosario):
    # conectar ca bd
    conexion = sqlite3.connect(DB_PATH + '/bd_glosarios_galego.bd')
    cursor = conexion.cursor()
    
    # obter da bd os id de concepto e traducion mais altos que haxa
    idConcepto = obterMaximo(cursor, "concepto")
    idTrad = obterMaximo(cursor, "traducion")
    
    # abrir o ficheiro TBX
    xmldoc = minidom.parse(ficheiroTBX)
    for concepto in xmldoc.getElementsByTagName("termEntry"):
        idConcepto += 1
        definicion = ""
        ambito = ""
        exemplo = ""
        
        # as etiquetas descrip poden ser de tipo "definition", "subjectField" ou "example"
        for descrip in concepto.getElementsByTagName("descrip"):
            tipoDescrip = descrip.getAttribute("type")
            if (tipoDescrip == "definition"):
                definicion = descrip.firstChild.data
            elif (tipoDescrip == "subjectField"):
                ambito = descrip.firstChild.data
            else:
                exemplo = descrip.firstChild.data
        
        # meter o idConcepto, o ambito, a definicion, o exemplo, o nomeGlosario na BD
        novoConcepto = (idConcepto, ambito, definicion, exemplo, nomeGlosario)
        cursor.execute('INSERT INTO concepto VALUES (?,?,?,?,?)', novoConcepto)
        conexion.commit()
        
        # agora percorremos os langSets (só hai dous, un do inglés e outro do galego)
        for langSet in concepto.getElementsByTagName("langSet"):
            # collemos o idioma do langSet actual
            idioma = langSet.getAttribute("xml:lang")
            
            # cada langSet ten tigs: percorrelos
            for tig in langSet.getElementsByTagName("tig"):
                idTrad += 1
                #<term>traducion</term>
                #<termNote type="administrativeStatus">preferredTerm-admn-sts</termNote>
                #<termNote type="partOfSpeech">preposition</termNote>
                #<termNote type="grammaticalGender">masculine</termNote>
                #<termNote type="grammaticalNumber">plural</termNote>
                #<termNote type="termType">abbreviation</termNote>
                traducion = tig.getElementsByTagName("term")[0].firstChild.data
                isRecommended = ""
                partOfSpeech = ""
                xenero = ""
                numero = ""
                
                for termNote in tig.getElementsByTagName("termNote"):
                    tipoNota = termNote.getAttribute("type")
                    if (tipoNota == "administrativeStatus"):
                        isRecommended = "true"
                    elif (tipoNota == "partOfSpeech" or tipoNota == "termType"):#só vin termType con abbreviation
                        partOfSpeech = termNote.firstChild.data
                    elif (tipoNota == "grammaticalGender"):
                        xenero = termNote.firstChild.data
                    else:
                        numero = termNote.firstChild.data
                
                #id_traducion, texto, idioma, is_recommended, partOfSpeech, grammaticalGender, numero, idConcepto
                novaTraducion = (idTrad, traducion, idioma, isRecommended, partOfSpeech, xenero, numero, idConcepto)
                cursor.execute('INSERT INTO traducion VALUES (?,?,?,?,?,?,?,?)', novaTraducion)
                conexion.commit()
                print ("Concepto: " + str(idConcepto) + "   Tradución: " + str(idTrad) + "   Glosario: " + nomeGlosario)
    conexion.commit()
    cursor.close()
    conexion.close()


if __name__ == "__main__":
    dicFicheiros = {# a orde afecta aos resultados que aparecerán antes nas consultas
        "01Trasno" : "trasno_glosario.tbx",
        "02Marce" : "terms_gl_marce.tbx",
        "03openSuse" : "vertaal_glosario_opensuse.tbx",
        "04XisTrasno" : "galego21_union_glosarios_xis_trasno.tbx",
        "05Xis" : "galego21_glosario_proxecto_xis.tbx",
        "06Netbeans" : "NetbeansGalicianGlossary.tbx",
        "07WikiMancomun" : "glosario_mancomun_wiki_novo.tbx",
        "08Mancomun" : "glosario_mancomun_09.tbx",
        "09OSLUDC" : "guia_estilo_OSLUDC.tbx",
        "10XustoRuibal" : "guia_estilo_xusto_ruibal_nomes_teclas.tbx",
        "11SNLInf" : "SNL_informatica.tbx",
        "12Ciberlingua" : "ciberlingua_terminoloxia.tbx",
        "13Guinovart" : "uvigo_informatica.tbx",
        "14Hermida" : "hermida_internet.tbx",
        "15Panlatino Internet" : "lexico_panlatino_internet.tbx",
        "16Panlatino comercio electronico" : "lexico_panlatino_comercio_electronico.tbx",
        "17Microsoft" : "ms_terminoloxia.tbx",
        "18San clemente" : "san_clemente_galego_ingles.tbx",
        "19Nova economia" : "termos_nova_economia.tbx",
        "20galego.org" : "galego.org_glosario_internet.tbx",
        "21OCEG_eAdministracion" : "OCEG_Glosario_da_eAdministracion.tbx",
        "22e-NegocioGalicia" : "glosario_e-negociogalicia.com.tbx",
        "23Siglas Anxo Forxan" : "siglas_anxo_forxan_sitio_Ramon_Flores.tbx",
        "24Siglas San Clemente" : "san_clemente_galego_ingles_siglas.tbx",
    }
    hIni = time.localtime()
    rutaBase = DB_PATH + "/glosarios_vocabularios_informatica_galego_TBX/"
    
    if ("bd_glosarios_galego.bd" in os.listdir(DB_PATH)):
        print ("\n\nA base de datos xa está creada.\n\n")
    else:
        crear_bd()
        print ("\n\nA base de datos non existia así que houbo que creala.\n\n")
    
    proxectos = dicFicheiros.keys()
    proxectos.sort()
    for clave in proxectos:
        #meterDatos(ficheiroTBX, nomeGlosario)
        meterDatos(rutaBase + dicFicheiros[clave], clave[2:])
    
    hFin = time.localtime()
    # cálculo do tempo arranxado
    minutos = hFin[4] - hIni[4]
    segundos = hFin[5] - hIni[5]
    if (segundos < 0):
        minutos = minutos - 1
        segundos = 60 + segundos
    print ("\n\n\tTardouse " + str(minutos) + " minutos, " + str(segundos) + " segundos\n\n")


